## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Than Ngo <than@redhat.com>
## Copyright (C) 2004 Leon Ho <llch@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

PROGNAME = 'system-switch-im'
TITLE = PROGNAME
IMSWITCHDIR = '/usr/share/' + PROGNAME + '/'
COPYRIGHT = 'Copyright (C) 2002-2004 Red Hat, Inc.'
AUTORS = ['Leon Ho <llch@redhat.com>']
VERSION = '0.1.2'
GLADEPATH = ''
TRUE = (1==1)
FALSE = not TRUE

import re
import traceback
import sys
import os
import os.path
import shutil
import string
import gettext
import glob

IIIMF = 0
XIM = 1

XINPUT_PATH = '/etc/X11/xinit/xinput.d/'
IM_PATH_PREFIX = '/etc/alternatives/xinput-'
ALT_DATA_PREFIX = '/var/lib/alternatives/xinput-'

## IMINFO class
class ImInfo:
    def __init__(self):
        self.alllangs = []
        self.langs = []
        self.iiimfentries = {}
        self.ximentries = {}
        self.imdefault = {}
        self.imchanged = {}
        self.iiimf = 0
        self.xim = 0
        self.initial = 0
       
        # Initialize and sort langs array with ??_?? in XINPUT_PATH 
        self.alllangs = glob.glob1(XINPUT_PATH, '??_??')
        self.alllangs.sort()
        # Use languages to initialize the default IM and available IMs
        for lang in self.alllangs:
            # check available IMs
            f = open(ALT_DATA_PREFIX + lang, 'r')
            strings = f.read()
            f.close()
            matched = re.findall('/etc/X11/xinit/xinput.d/([a-zA-Z0-9\-]*)\n', strings)

            # if more than one options of IM for that language, list the lang
            if len(matched) > 1:
               self.langs.append(lang)

            self.iiimfentries[lang] = []
            self.ximentries[lang] = []

            for match in matched:
                if match == 'iiimf':
                    self.iiimfentries[lang].append(match)
                    self.iiimf = 1
                else:
                    self.ximentries[lang].append(match)
                    self.xim = 1

            # check set IM
            imdefaultlink = IM_PATH_PREFIX + lang
            if os.path.exists(imdefaultlink) and os.path.islink(imdefaultlink):
              matched = re.findall('/etc/X11/xinit/xinput.d/([a-zA-Z0-9\-]*)', 
                      os.readlink(imdefaultlink))
              self.imdefault[lang] = matched[0]

            self.imchanged = self.imdefault

        # Get the actual names for the locales
        if os.path.isfile('locale-list'):
            localefile = 'locale-list'
        if os.path.isfile(IMSWITCHDIR + 'locale-list'):
            localefile = IMSWITCHDIR + 'locale-list'

        f = open(localefile, 'r')
        self.langnames = {}
        string = "."
        while string != "":
            string = f.readline()
            lang = re.search('^(.._..)*', string).group()
            name = re.findall('^.*sun16 (.*)$', string)
            if lang != "":
                  self.langnames[lang] = name[0]

    # Returns if IIIMF is available
    def is_iiimf(self):
        return self.iiimf

    # Returns if XIM is available
    def is_xim(self):
        return self.xim
            
    # Returns available languages
    def get_langs(self):
        return self.langs

    # Returns IIIMF IMs for langs
    def get_iiimfentries(self):
        return self.iiimfentries

    # Returns XIM IMs for langs
    def get_ximentries(self):
        return self.ximentries
        
    # Returns the default IM for each lang
    def get_imdefault(self):
        return self.imdefault

    # Returns the default IM for specific lang
    def get_imlangdefault(self, lang):
        return self.imdefault[lang]

    # Returns the language name of the locale
    def get_langname(self, lang):
        return self.langnames[lang]

    # Set IIIMF or XIM for all languages
    def set_allimdefault(self, framework):
        if framework == IIIMF:
            for lang in self.langs:
                os.system('/usr/sbin/alternatives --set xinput-' + lang + ' /etc/X11/xinit/xinput.d/' + self.iiimfentries[lang][0] + ' 2>/dev/null >/dev/null')
        elif framework == XIM:
            for lang in self.langs:
                if self.ximentries[lang]:
                    os.system('/usr/sbin/alternatives --set xinput-' + lang + ' /etc/X11/xinit/xinput.d/' + self.ximentries[lang][0] + ' 2>/dev/null >/dev/null')

    # Returns changes from the user on language
    def get_changed_im(self, lang):
        return self.imchanged[lang]

    # Set changes from user on the language
    def set_changed_im(self, lang, im):
        if self.imchanged[lang]:
            self.imchanged[lang] = im
            return TRUE
        else:
            return FALSE
            
    # Commit changes to system from user changes
    def commit_imdefault(self):
        for lang in self.langs:
            os.system('/usr/sbin/alternatives --set xinput-' + lang + ' /etc/X11/xinit/xinput.d/' + self.imchanged[lang] + ' 2>/dev/null >/dev/null')

    def get_changed_im_pos(self, lang):
        i = 0
        for entry in self.iiimfentries[lang]:
            if entry == self.imchanged[lang]:
                return i
            i = i + 1
        for entry in self.ximentries[lang]:
            if entry == self.imchanged[lang]:
                return i
            i = i + 1
        return 0

    # Returns the current default framework
    def get_default_framework(self):
        im_val = 0
        im_total = 0
    
        for lang in self.langs:
            if self.get_imlangdefault(lang) == 'iiimf':
                im_val = im_val + 1
            im_total = im_total + 1    

        if im_val == im_total:
            return 'IIIMF'
        elif im_val == 0:
            return 'XIM'
        else:
            return 'AD'


##
## I18N
##
gettext.bindtextdomain(PROGNAME, '/usr/share/locale')
gettext.textdomain(PROGNAME)
_ = gettext.gettext

WARNING_TEXT = _('There are no input method system installed on system.\n'
                 'Please install it and restart this tool again.')

INFO_TEXT = _("This software is distributed under the GPL. Please Report bugs to Red Hat's Bug Tracking System: http://bugzilla.redhat.com/")


INFO_TEXT1 = _('Your input method configuration has successfully been updated.\n'
               'Please restart X to take effect.')

TEXT = _('The Input Method Switcher is a tool which enables users to easily switch between various input method system that they have installed.\n\nPlease choose your input method system - IIIMF, or use advanced settings to customize IIIMF or XIM for each different languages.')

IM_TEXT = _('Input Methods')

LANG_TEXT = _('Languages')
