#! /usr/bin/env python
## system-switch-im - A input method switching tool
## Copyright (C) 2002 Red Hat, Inc.
## Copyright (C) 2002 Than Ngo <than@redhat.com>
## Copyright (C) 2004 Leon Ho  <llch@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

IMSWITCHDIR = '/usr/share/system-switch-im/'

import sys

if not "/usr/lib/rhs/python" in sys.path:
    sys.path.append("/usr/lib/rhs/python")

if not IMSWITCHDIR in sys.path:
    sys.path.append(IMSWITCHDIR)

import signal
import os
import os.path
import string
import gettext
from functions import *


##
## I18N
##
gettext.bindtextdomain(PROGNAME, '/usr/share/locale')
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, '/usr/share/locale', 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode    

import gtk
import gtk.glade

class mainDialog:
    def __init__(self):
        pass

    def main(self):
        glade_file = 'imswitch.glade'

        self.iminfo = ImInfo()

        if not os.path.isfile(glade_file):
            glade_file = GLADEPATH + glade_file
        if not os.path.isfile(glade_file):
            glade_file = IMSWITCHDIR + glade_file

        self.xml = gtk.glade.XML(glade_file, None, domain=PROGNAME)

        self.xml.signal_autoconnect(
            {
            'on_cancelB_clicked' : self.on_cancelB_clicked,
            'on_okB_clicked' : self.on_okB_clicked,
            'on_advancedRB_toggled' : self.on_advancedRB_toggled,
            'on_langTV_cursor_changed' : self.on_langTV_cursor_changed,
            'on_imCBentry_changed' : self.on_imCBentry_changed,
            })

        self.dialog = self.xml.get_widget('window')
        self.iiimfRB = self.xml.get_widget('iiimfRB')
        self.ximRB = self.xml.get_widget('ximRB')
        self.advancedRB = self.xml.get_widget('advancedRB')

        self.langtreeview = self.xml.get_widget('langTV')
        self.langscrollwindow = self.xml.get_widget('langSW')

        self.imCB = self.xml.get_widget('imCB')
        
        self.langtreestore = gtk.TreeStore(str, str)
        self.langtreeview.set_model(self.langtreestore)
        self.langtvcolumn = gtk.TreeViewColumn(LANG_TEXT)
        self.langtreeview.append_column(self.langtvcolumn)
        self.langcell = gtk.CellRendererText()
        self.langtvcolumn.pack_start(self.langcell, True)
        self.langtvcolumn.add_attribute(self.langcell, 'text', 0)


        #self.langtvcolumnim = gtk.TreeViewColumn('Input Method')
        #self.langtreeview.append_column(self.langtvcolumnim)
        #self.langcellim = gtk.CellRendererText()
        #self.langtvcolumnim.pack_start(self.langcellim, True)
        #self.langtvcolumnim.add_attribute(self.langcellim, 'text', 0)

        for lang in self.iminfo.get_langs():
            self.langtreestore.append(None, [self.iminfo.get_langname(lang), lang])
    
        
        self.adSensitive()
        
        self.dialog.connect('delete-event', self.on_Dialog_delete_event)
        self.dialog.connect('hide', gtk.main_quit)
        #self.load_icon('imswitch-mini.xpm', self.dialog)
        self.xml.get_widget('imPixmap').set_from_file(IMSWITCHDIR + 'pixmaps/imswitch.png')

        self.hydrate()
        gtk.main()
        
    def on_Dialog_delete_event(self, *args):
        gtk.main_quit()
    
    def on_cancelB_clicked(self, button):
        gtk.main_quit()

    def on_okB_clicked(self, button):
        if self.iiimfRB.get_active():
            if self.iminfo.initial != 'IIIMF':
                self.iminfo.set_allimdefault(IIIMF)
                self.infoDialog()
        elif self.ximRB.get_active():
            if self.iminfo.initial != 'XIM':
                self.iminfo.set_allimdefault(XIM)
                self.infoDialog()
        elif self.advancedRB.get_active():
            self.iminfo.commit_imdefault()
            self.infoDialog()
                

        #self.dialog.hide()
        gtk.main_quit()

    def on_advancedRB_toggled(self, *args):
        self.adSensitive()
    
    def adSensitive(self):
        activate = self.advancedRB.get_active()
        self.langtreeview.set_sensitive(activate)
        self.langscrollwindow.set_sensitive(activate)
        self.imCB.set_sensitive(activate)

    def on_langTV_cursor_changed(self, *args):
        list = []
        langTS = self.langtreeview.get_selection()
        model, iter = langTS.get_selected()
    
        lang = self.langtreestore.get_value(iter, 1)

        imentries = self.iminfo.get_iiimfentries()
        for entry in imentries[lang]:
            list.append(entry)
        imentries = self.iminfo.get_ximentries()
        for entry in imentries[lang]:
            list.append(entry)

        im = self.iminfo.get_changed_im(lang)
        self.imCB.set_popdown_strings(list)
        self.imCB.entry.set_text(im)
    
    def on_imCBentry_changed(self, *args):
        langTS = self.langtreeview.get_selection()
        model, iter = langTS.get_selected()
        lang = self.langtreestore.get_value(iter, 1)

        im = self.imCB.entry.get_text()

        if self.imCB.entry.get_text() != '':
            self.iminfo.set_changed_im(lang, im)
        
    
    def hydrate(self):
        self.xml.get_widget('imLabel').set_text(TEXT)
        self.xml.get_widget('imselLabel').set_label(IM_TEXT)
        self.xml.get_widget('frame').set_label(_('Available Input Method System'))

        if (self.iminfo.is_iiimf() and self.iminfo.is_xim) == FALSE:
            self.warningDialog()
            sys.exit(1)

        # show im checkbox
        if self.iminfo.is_iiimf() == TRUE:
            self.iiimfRB.show()
        # XXX disabled XIM selection
        #if self.iminfo.is_xim() == TRUE:
        #    self.ximRB.show()

        self.advancedRB.show()

        # activate default im
        im = self.iminfo.get_default_framework()
        if im == 'IIIMF':
            self.iiimfRB.set_active(TRUE)
            self.iminfo.initial = 'IIIMF'
        elif im == 'XIM':
            self.ximRB.set_active(TRUE)
            self.iminfo.initial = 'XIM'
        elif im == 'AD':
            self.advancedRB.set_active(TRUE)
            self.iminfo.initial = 'AD'

    def infoDialog(self):
        dlg = gtk.MessageDialog(self.dialog,
                                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,                                
                                gtk.MESSAGE_INFO,
                                gtk.BUTTONS_OK,
                                INFO_TEXT1)
        
        dlg.set_position (gtk.WIN_POS_MOUSE)
        dlg.run()
        dlg.destroy()

    def warningDialog(self):
        dlg = gtk.MessageDialog(self.dialog,
                                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                gtk.MESSAGE_INFO,
                                gtk.BUTTONS_OK,
                                WARNING_TEXT)
        dlg.set_position (gtk.WIN_POS_MOUSE)
        dlg.run()
        dlg.destroy()

    def get_pixbuf(self, pixmap_file):
        fn = pixmap_file
        if not os.path.exists(pixmap_file):
            pixmap_file = "pixmaps/" + fn
            if not os.path.exists(pixmap_file):
                pixmap_file = "../pixmaps/" + fn
                if not os.path.exists(pixmap_file):
                    pixmap_file = IMSWITCHDIR + fn
                    if not os.path.exists(pixmap_file):
                        pixmap_file = IMSWITCHDIR + "pixmaps/" + fn
                        if not os.path.exists(pixmap_file):
                            return None, None

        pixbuf = gtk.gdk.pixbuf_new_from_file(pixmap_file)

        return pixbuf

    def get_icon(self, pixmap_file, dialog):
        pixbuf = self.get_pixbuf(pixmap_file)
    
        pix, mask = pixbuf.render_pixmap_and_mask()
    
        return pix, mask


    def load_icon(self, pixmap_file, dialog):
        if not dialog: return
        
        pixbuf = self.get_pixbuf(pixmap_file)
        if not pixbuf: return
     
        if dialog: dialog.set_icon(pixbuf)

    def rootWarning(self, errMsg):
        dlg = gtk.MessageDialog(None,
                                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                gtk.MESSAGE_INFO,
                                gtk.BUTTONS_OK,
                                errMsg)
        dlg.set_position (gtk.WIN_POS_MOUSE)
        dlg.run()
        dlg.destroy()
        
# make ctrl-C work
if __name__ == '__main__':
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    dlg = mainDialog().main()
    
    sys.exit(0)
